/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: deficon.c,v 1.5 2006/04/12 14:01:53 fab Exp $
 */

#include "ambient.h"

#if USE_DEFICONS

/* public */
#include <dos/filehandler.h>
#include <proto/dos.h>
#include <proto/exec.h>

/* private */
#include "deficon_getpath.h"
#include "deficon.h"
#include "file_func.h"
#include "device_func.h"
#include "name.h"

static void kill_digits(STRPTR s)
{
	ASSERT(s);

	while (*s)
	{
		if (*s >= '0' && *s <= '9')
		{
			*s = 'x';
		}
		s++;
	}
}


/*
 * Returns the dostype name.
 */
static ULONG devdostype(struct DosList *dl)
{
	struct FileSysStartupMsg *fssm;
	ULONG *dosenvec;

	if (dl)
	{
		fssm = BADDR(dl->dol_misc.dol_handler.dol_Startup);
		if (isfssm(fssm))
		{
			dosenvec = BADDR(fssm->fssm_Environ);
			if (dosenvec && dosenvec[DE_TABLESIZE] >= DE_DOSTYPE)
			{
				return (dosenvec[DE_DOSTYPE]);
			}
		}
	}
	return (0);
}

STRPTR deficon_build_devicename(STRPTR filename)
{
	struct MsgPort *mp;
	TEXT path[PATH_SIZE + 64];
	BPTR l = NULL;

	THREAD;
	ASSERT(filename);

	if ( (mp = DeviceProc(filename)) )
	{
		struct DosList *dl;
		ULONG dostype = NULL;
		UBYTE devname[32 + 1];
		UBYTE *buf;

		/* note: that 64 is left for the actual filename */
		if (!deficon_getpath(path, PATH_SIZE, "def_"))
		{
			/* would overflow anyway, so don't bother */
			return NULL;
		}
		buf = path + strlen(path);

		devname[0] = '\0';

		D(DEFICON, bug("locking list..\n"));

		dl = LockDosList(LDF_DEVICES | LDF_READ);
		while ( (dl = NextDosEntry(dl, LDF_DEVICES)) )
		{
			if (dl->dol_Task == mp)
			{
				CONST_STRPTR p = BADDR(dl->dol_Name);
				if (!p || p[0] >= 32)
					break;

				memcpy(devname, p + 1, p[0]);
				devname[p[0]] = '\0';

				sprintf(buf, "%sdisk.info", devname);
				dostype = devdostype(dl);
				D(DEFICON, bug("dostype: %ld\n", dostype));
				break;
			}
		}
		D(DEFICON, bug("unlocking list..\n"));
		UnLockDosList(LDF_DEVICES | LDF_READ);

		if (buf[0])
		{
			D(DEFICON, bug("computed name: %s\n", path));
		}

		if (buf[0] == '\0' || !(l = Lock(path, ACCESS_READ)))
		{
			/*
			 * Not found. Try with a more generic name:
			 * (ie. PC2disk -> PCxdisk).
			 */

			kill_digits(buf);
			D(DEFICON, bug("trying %s..\n", path));

			if (buf[0] == '\0' || !(l = Lock(path, ACCESS_READ)))
			{
				/*
				 * Still not found. Try something like
				 * def_MSD0disk.
				 * XXX: a dostype can be NULL! eg. Barthel's smbfs, ram. handle that
				 */
				UBYTE dt[5];

				dostype_to_str(dostype, dt);
				sprintf(buf, "%sdisk.info", dt);

				D(DEFICON, bug("trying %s..\n", path));

				if (!(l = Lock(path, ACCESS_READ)))
				{
					/*
					 * Gnah, try a more generic like def_MSDxdisk.
					 */
					kill_digits(buf);
					D(DEFICON, bug("trying %s..\n", path));
					 
					if (!(l = Lock(path, ACCESS_READ)))
					{
						BOOL def_disk_tried = FALSE;

						/*
						 * Sigh. Try to detect if it's a floppy.
						 * In that case, get disk.info.
						 */
						if (devname[0] == 'D' && devname[1] == 'F' &&
						    devname[2] >= '0' && devname[2] <= '9' && /* NOTE: catweasel might be mounted as DF4: for example */
						    devname[3] == '\0')
						{
							strcpy(buf, "disk.info");

							D(DEFICON, bug("oh, it's a floppy disk, trying %s..\n", path));
							if ( (l = Lock(path , ACCESS_READ)) )
							{
								goto skip;
							}
							def_disk_tried = TRUE;
						}

						/*
						 * Sigh. Try def_device.info..
						 */
						strcpy(buf, "device.info");

						D(DEFICON, bug("trying %s..\n", path));

						if (!(l = Lock(path, ACCESS_READ)))
						{
							/*
							 * Damn. What a crappy system. Try def_disk.info
							 * then give up.
							 */
							if (!def_disk_tried)
							{
								strcpy(buf, "disk.info");

								D(DEFICON, bug("holy fuck, trying %s..\n", path));
								l = Lock(path, ACCESS_READ);
							}
						}
					}
				}
			}
		}
	}

	skip:

	if (l)
	{
		/* nice buffer re-use here */
		if (NameFromLock(l, path, sizeof(path)))
		{
			UnLock( l );

			return name_build(path);
		}
		else
		{
			UnLock( l );
		}

		/* XXX */
	}

	return (NULL);
}


void deficon_delete_devicename(STRPTR s)
{
	name_delete(s);
}

#endif /* USE_DEFICONS */
